@@ -60,7 +60,7 @@ def filter_type(
 
 
 def generate_tmap_config(
-    filename: str, layer_data: List[FullLayerData]
+    filename: str, layer_data: List[FullLayerData], internal_shapes: bool = False
 ) -> Dict[str, Any]:
     """Generates a dict containing the tissumaps cfg of the napari layers to be saved.
 
@@ -72,6 +72,9 @@ def generate_tmap_config(
         The layers to be saved as provided by the Napari plugin manager. It contains a
         list of layers, which are themselves dictionary containing the data, the meta-
         data and the type of layer.
+    internal_shapes : bool
+        Determines if the shapes layer are saved in the tmap file (True) or if the tmap
+        file references an external json file (False).
     Returns
     -------
     Dict[str, Any]
@@ -104,7 +107,6 @@ def generate_tmap_config(
     # Generating the list of layers (images and labels)
     layers, layer_filters, layer_opacities, layer_visibilities = [], {}, {}, {}
     default_filters = [
-        {"name": "Saturation", "value": "0"},
         {"name": "Brightness", "value": "0"},
         {"name": "Contrast", "value": "1"},
     ]
@@ -154,22 +156,21 @@ def generate_tmap_config(
         "compositeMode": "lighter",
         "filename": filename,
         "layers": layers,
-        "filters": ["Saturation", "Brightness", "Contrast", "Color"],
+        "filters": ["Brightness", "Contrast", "Color"],
         "layerFilters": layer_filters,
         "layerOpacities": layer_opacities,
         "layerVisibilities": layer_visibilities,
         "markerFiles": markers,
-        "regions": regions,
         "settings": [
-            {
-                "function": "_linkMarkersToChannels",
-                "module": "overlayUtils",
-                "value": True,
-            },
             {"function": "_autoLoadCSV", "module": "dataUtils", "value": True},
-            {"function": "_markerScale2", "module": "glUtils", "value": 7.5},
+            {"function": "_globalMarkerScale", "module": "glUtils", "value": 7.5},
         ],
     }
+    if internal_shapes:
+        config["regions"] = regions
+    else:
+        config["regionFile"] = "regions/regions.json"
+
     return config
 
 
@@ -194,17 +195,50 @@ def generate_shapes_dict(data: FullLayerData, meta: Dict[str, Any]) -> Dict[str,
     shape_dict = {}
     for i, shape in enumerate(data):
         shape_name = meta["name"] + f"_{i+1}"
+        shape_type = meta["shape_type"][i]
         # We enumerate each shapes that appear in the layer
         subshape_dict = {}
         subshape_dict["id"] = shape_name
+        # Different shapes have different points to draw
+        points_to_draw = []
+        if shape_type == "ellipse":
+            assert isinstance(shape, np.ndarray)
+            ellipse_center = (
+                (shape[0][0] + shape[2][0]) / 2.,
+                (shape[0][1] + shape[1][1]) / 2.,
+            )
+            # `a` represents the vector from the center of the ellipse to the right
+            # hand side, while `b` is the up vector.
+            ellipse_a = shape[1][0] - ellipse_center[0]
+            ellipse_b = shape[1][1] - ellipse_center[1]
+
+            # Minimum arc distance is the the length of a single arc as a function of
+            # the ellipse's radii. The purpose is such that the resolution (number of
+            # points) grows with the ellipse. The formula is approximated and computes
+            # the arc based on a circle with the radius being equal to the longest
+            # axis of the ellipse.
+            minimum_arc_distance = 3.
+            max_axis = np.maximum(np.abs(ellipse_a), np.abs(ellipse_b))
+            N = int(np.ceil(2. * np.pi * max_axis / minimum_arc_distance))
+            thetas = np.linspace(0, 2*np.pi, N+1)
+            points_to_draw = np.stack([
+                ellipse_a * np.cos(thetas) + ellipse_center[0],
+                ellipse_b * np.sin(thetas) + ellipse_center[1]
+            ], axis=-1)
+        elif shape_type == "line" or shape_type == "path":
+            assert isinstance(shape, np.ndarray)
+            points_to_draw = np.vstack([shape, shape[-2::-1]])
+        else: # shape_type == "polygon" or shape_type == "rectangle"
+            assert isinstance(shape, np.ndarray)
+            points_to_draw = shape
+
         # Points with pixel positions
         points_array = []
         _xmin, _xmax, _ymin, _ymax = np.inf, -np.inf, np.inf, -np.inf
         dimensions = current_viewer().dims.range
         width, height = dimensions[0][1], dimensions[1][1]
-        for _points in shape:
-            assert isinstance(_points, np.ndarray)
-            points = [_points[0] / width, _points[1] / height]
+        for _points in points_to_draw:
+            points = [_points[1] / height, _points[0] / width]
             points_array.append({"x": points[0], "y": points[1]})
             _xmin = _xmin if points[0] > _xmin else points[0]
             _xmax = _xmax if points[0] < _xmax else points[0]
@@ -214,8 +248,8 @@ def generate_shapes_dict(data: FullLayerData, meta: Dict[str, Any]) -> Dict[str,
         # Points with normalized positions (in [0,1])
         global_points_array = []
         _gxmin, _gxmax, _gymin, _gymax = np.inf, -np.inf, np.inf, -np.inf
-        for points in shape:
-            assert isinstance(points, np.ndarray)
+        for _points in points_to_draw:
+            points = [_points[1], _points[0]]
             global_points_array.append({"x": points[0], "y": points[1]})
             _gxmin = _gxmin if points[0] > _gxmin else points[0]
             _gxmax = _gxmax if points[0] < _gxmax else points[0]
@@ -258,7 +292,7 @@ def rgb2hex(color_vec: np.ndarray) -> str:
     str
         The color as a string in hex format.
     """
-    return "#" + "".join([f"{int(c*255):2X}" for c in color_vec[:3]])
+    return "#" + "".join([f"{int(c*255):02X}" for c in color_vec[:3]])
 
 
 def tmap_writer(
@@ -293,6 +327,9 @@ def tmap_writer(
     tmap_file = open(save_path / "main.tmap", "w+")
     tmap_file.write(json.dumps(tmap_cfg, indent=4))
     tmap_file.close()
+
+    # Shapes have to be combined in the same file
+    regions = {}
     # Saving the files
     for data, meta, layer_type in layer_data:
         if layer_type == "image":
@@ -336,18 +373,21 @@ def tmap_writer(
                 label_img_uint8 = (label_img * 255.0).astype(np.uint8)
                 imsave(str(path_label), label_img_uint8)
         elif layer_type == "shapes":
-            shapes_folder = save_path / "shapes"
-            shapes_folder.mkdir(exist_ok=True)
-
-            # Saving the json
-            shapes_filename = meta["name"] + ".json"
-            shapes_file = open(shapes_folder / shapes_filename, "w+")
-            shape_dict = generate_shapes_dict(data, meta)
-            shapes_file.write(json.dumps(shape_dict, indent=4))
-            shapes_file.close()
+            regions.update(generate_shapes_dict(data, meta))
         else:
             logging.warning(
                 f"Layer \"{meta['name']}\" cannot be saved. This type of layer "
                 "({layer_type}) is not yet implemented."
             )
+
+    # Saving the shapes
+    if len(regions) > 0:
+        shapes_folder = save_path / "regions"
+        shapes_folder.mkdir(exist_ok=True)
+        # Saving the json
+        shapes_filename = "regions.json"
+        shapes_file = open(shapes_folder / shapes_filename, "w+")
+        shapes_file.write(json.dumps(regions, indent=4))
+        shapes_file.close()
+
     return str(save_path)
